<?php

/*
 * @version;   						2.6
 * @category;						widget
 * @copyright;   					Copyright (C) 2017 Chatwee Team. All rights reserved.
 * @license;   						GNU GPLv3 http://www.gnu.org/licenses/gpl.html
 * @link ;   						https://www.chatwee.com/
 */

defined( '_JEXEC' ) or die( 'Direct Access Denied' );

jimport( 'joomla.plugin.plugin');

jimport( 'joomla.html.parameter');

jimport('joomla.log.log');

JLog::addLogger(array());

require_once(dirname( __FILE__ ) . "/ChatweeV2_SDK/Chatwee.php");

class plgSystemChatwee extends JPlugin {

    function __construct(&$subject, $config)	{
        parent::__construct($subject, $config);
        $this->_plugin = JPluginHelper::getPlugin("system", "chatwee");
        $this->_params = new JRegistry( $this->_plugin->params );
    }

    function onAfterRender() {
        $panel_admin = JFactory::getApplication();
        $widget_code = $this->params->get('chatwee_code');

        if($widget_code == '' || $panel_admin->isAdmin() || strpos($_SERVER["PHP_SELF"], "index.php") === false) {
            return;
        }

        $document = JFactory::getDocument();
        $block = JResponse::getBody();
        $block = str_replace("</body>", $widget_code . "</body>", $block);
        JResponse::setBody($block);

        return true;
    }

    function onUserAfterLogin() {
        $ssoStatus = $this->params->get('chatwee_sso_status');

        if($ssoStatus == 1) {
            $user = JFactory::getUser();
            $this->sso_login($user);
        }
    }

    function onUserLogout()	{
        $ssoStatus = $this->params->get('chatwee_sso_status');

        if($ssoStatus == 1) {
            $this->sso_logout();
        }
    }

    function onAfterInitialise() {
        $ssoStatus = $this->params->get('chatwee_sso_status');

        if($ssoStatus == 1) {
            $this->sso_logout();
        }
        $this->synchronize_sso_user();
    }

    function set_sso_credentials() {
        $chatApiUrl = $this->params->get('chatwee_chatapiurl');
        $chatId = $this->params->get('chatwee_chatid');
        $clientKey = $this->params->get('chatwee_apikey');

        ChatweeV2_Configuration::setChatApiUrl($chatApiUrl);
        ChatweeV2_Configuration::setChatId($chatId);
        ChatweeV2_Configuration::setClientKey($clientKey);
        ChatweeV2_Configuration::setCustomUserAgent("Chatwee Joomla Plugin 2.6");
    }

    function synchronize_sso_user() {
        $ssoStatus = $this->params->get("chatwee_sso_status");

        if(!empty($_SERVER["HTTP_X_REQUESTED_WITH"]) && strtolower($_SERVER["HTTP_X_REQUESTED_WITH"]) === "xmlhttprequest") {
            return false;
        }

        if($ssoStatus == 1) {

            $user = JFactory::getUser();
            if(!$user->id) {
                return false;
            }

            $this->set_sso_credentials();

            try {
                if(ChatweeV2_Session::isSessionSet() === false) {
                    $this->sso_login($user);
                } else {
                    $sessionId = ChatweeV2_Session::getSessionId();

                    $isSessionValid = ChatweeV2_SsoUser::validateSession(Array(
                        "sessionId" => $sessionId
                    ));

                    if($isSessionValid === false) {
                        $this->sso_login($user);
                    }
                }
            } catch(Exception $exception) {
                JLog::add("Error while reinitializng SSO session: " . $exception->getMessage());
            }
        }
    }

    function sso_login($user) {
        $this->set_sso_credentials();

        $user_chatwee_id = $this->fetch_user_chatwee_id($user);

        if(!$user_chatwee_id) {
            $user_chatwee_id = $this->sso_register($user);
        }

        try	{
            ChatweeV2_SsoUser::edit(Array(
                "userId" => $user_chatwee_id,
                "login" => $user->name ? $user->name : $user->username,
                "avatar" => "",
                "isAdmin" => $this->is_chatwee_admin($user)
            ));
        } catch(Exception $exception) {
            if(strpos($exception->getMessage(), "User does not exist") !== false) {
                $user_chatwee_id = $this->sso_register($user);
            } else {
                JLog::add("Error while executing sso_login: " . $exception->getMessage());
            }
        }

        try {
            ChatweeV2_SsoManager::loginUser(Array(
                "userId" => $user_chatwee_id
            ));
        } catch(Exception $exception) {
            JLog::add("Error while executing sso_login: " . $exception->getMessage());
        }
    }

    function sso_logout() {
        $this->set_sso_credentials(2);
        try {
            ChatweeV2_SsoManager::logoutUser();
        } catch(Exception $exception) {
            JLog::add("Error while executing logout: " . $exception->getMessage());
        }
    }

    function is_chatwee_admin($user) {
        if(!$user) {
            $user = JFactory::getUser();
        }
        $groups = $user->get('groups');
        $adminGroups = $this->params->get('chatwee_allowed_groups');
        if(isset($adminGroups)) {
            foreach ($groups as $group) {
                if (in_array($group, $adminGroups)) {
                    return true;
                }
            }
        }
        return false;
    }

    function fetch_user_chatwee_id($user) {
        $username = $user->username;

        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query->select($db->quoteName(array('chatwee_id')));
        $query->from($db->quoteName('#__chatwee_users_data'));
        $query->where($db->quoteName('username') . ' LIKE '. $db->quote($username));
        $db->setQuery($query);

        $result = $db->loadResult();
        return $result ? $result : null;
    }

    function sso_register($user) {

        try {
            $userId = ChatweeV2_SsoManager::registerUser(Array(
                "login" => $user->name ? $user->name : $user->username,
                "avatar" => "",
                "isAdmin" => $this->is_chatwee_admin($user)
            ));

            $db = JFactory::getDbo();
            $query = $db->getQuery(true);
            $query->delete($db->quoteName('#__chatwee_users_data'));
            $query->where($db->quoteName('username') . ' LIKE '. $db->quote($user->username));
            $db->setQuery($query);
            $result = $db->execute();

            $profile = new stdClass();
            $profile->username = $user->username;
            $profile->chatwee_id = $userId;
            $result = JFactory::getDbo()->insertObject('#__chatwee_users_data', $profile);

            return $userId;
        } catch(Exception $exception) {

            JLog::add("Error while executing sso_register: " . $exception->getMessage());
            return null;
        }
    }
}

?>